/*
**	  GPU Raytrace - Diploma thesis, Czech technical university
**	  Copyright (C) 2008 Martin Zlatuska
**
**	  This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <vector_types.h>

#include "Utils/SPMatrix.h"
#include "Utils/SseUtils.h"

#include "CameraCuda.h"

//! Generates rays for ray casting 
class Camera:public AllignedS  {
public:
	unsigned pixelWidth;			//!< Number of rays to generate per line
	unsigned pixelHeight;			//!< Number of ray's lines to generate
	float	 planeWidth;			//!< Half width size of projection plane
	float	 planeHeight;			//!< Half height size of projection plane

	SPVector upperLeft;				//!< The direction of the upper left ray(first ray for tracing)
	SPVector dx;					//!< Horizontal difference betwen rays
	SPVector dy;					//!< Vertical difference betwen rays

public:
	SPVector eye;					//!< Center of projection

	SPVector moveFront;
	SPVector moveSide;

public:
	/*! */  
	Camera(void) {}; 
	/*! */
	~Camera(void) {}; 

	//! Sets camera parameters
	/*!
	* @param newWidth Number of rays to generate per line
	* @param newHeight Number of ray's lines to generate
	* @param fov Field of view
	*/
	void setUpProjection(unsigned newWidth,unsigned newHeight,float fov);

	//! Sets up new camera position and orientation
	/*!
	* @param newEye New center of projection
	* @param target Point which will be seeen in center of image
	* @param up Upvector defining camera rotation around direction of look
	*/
	void setLook(const SPVector &newEye, const SPVector &target, const SPVector &up);

	//! Copy the values to the structure that can be passed to CUDA functions
	/**
	*	@param[out] cameraOut	The target for the values.
	*/
	void prepareCuda(CameraCuda &cameraOut);

};

