/*
**	  GPU Raytrace - Diploma thesis, Czech technical university
**	  Copyright (C) 2008 Martin Zlatuska
**
**	  This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#pragma once

#include "FileLoader.h"
#include "Lexan.h"

#include "Material.h"

#include "Utils/Memory.h"
#include "Geometry.h"

class InventorReader:public FileLoader,public Lexan {
	class UnexpectedSymbol {};

	struct InventorMaterial {
		float3  ambient;
		float3	diffuse;
		float3  specular;
		float3  emmisive;
		float   shinines;
		float	reflectivity;
		float	transparency;

		void reset();
	};

	struct SubObject {
		unsigned vertexStart;
		unsigned vertexCount;
		unsigned normalStart;
		unsigned coorStart;
		unsigned faceCount;
		unsigned normStart;
	
		InventorMaterial material;
	};

	InventorMaterial currentMaterial;

	GrowArray<float4>	*verticesDataG; 
	GrowArray<float4>	*normalsDataG;
	GrowArray<unsigned> *coordDataG;
	GrowArray<unsigned> *normDataG;

	std::vector<SubObject> subObjects;
	SubObject*			   currentObject;
	

	Token token;

	bool is(Token check) {return token == check;};
	void check(Token test) {if(token != test) throw UnexpectedSymbol();};
	void nextToken(bool wantInteger=true) {token = getToken(wantInteger);};

	void parseColor(float3 &dst);

	void coordIndex();
	void normalIndex();
	void materialBinding();
	void normalBinding();
	void normals();
	void geometry();
	void vertices();
	void mesh();
	void material();
	void shapeHints();
	void main();

	Object* makeObject();
public:
	InventorReader(void);
	~InventorReader(void);

	virtual Object* loadFile(const char *fileName);
};
