/*
**	  GPU Raytrace - Diploma thesis, Czech technical university
**	  Copyright (C) 2008 Martin Zlatuska
**
**	  This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Lexan.h"

Token Lexan::getToken(bool wantInteger) {
    eatWhite();		//Skip whitespace characters.
    char ch=*text;

	if(ch==0) return LEX_EOF;
	if(ch == '#') {	//The rest of the line is comment
		while(ch == '#' && *text!=0 ) {	//Skip all the lines with the comments.
			text = strchr(text,'\n');
			if(text)
				ch=*text;
			else
				return LEX_EOF;
		};

		eatWhite();
		ch=*text;
	};

	//Check for special function characters.
	switch(ch) {
		case '{': text++; return LEX_OPEN_CURLY;  break;
		case '}': text++; return LEX_CLOSE_CURLY; break;
		case '[': text++; return LEX_OPEN_BRACKET; break;
		case ']': text++; return LEX_CLOSE_BRACKET; break;
		case ';': text++; return LEX_SEMICOLON; break;
		case ',': text++; return LEX_COMMA; break;
	};
	if(isNumber(ch)) {	//The next element is number.
		char *space = strchr(text,' ');

		//Use hint from upper part of hierarchy to parse right type of number.
		if(wantInteger) 
			redInteger = strtol(text,&text,10);
		else
			redFloat   = strtod(text,&text);

		*space = ' ';

		return LEX_NUMBER;
	};
	if(isTextChar(ch)) { //The next element is keyword. Try to identify it and return it's code.
		char* wordEnd=text+1;
		while(isTextChar(*wordEnd)) wordEnd++;

		char save = *wordEnd;
		*wordEnd = 0;

		Token token;

		map<string,Token>::iterator findToken=tokenMap.find(text);
		if(findToken != tokenMap.end())
			token = findToken->second;
		else
			throw UnknownKeyword();

		*wordEnd = save;
		text     = wordEnd;

		return token;
	};

	throw UnknownKeyword();
};
