/*
**	  GPU Raytrace - Diploma thesis, Czech technical university
**	  Copyright (C) 2008 Martin Zlatuska
**
**	  This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "MyLoader.h"

#include "Transformations/SceneNode.h"

static const unsigned addDummy = SceneLoader::instance()->addLoader(new MyLoader,"mdl");


MyLoader::MyLoader(void) {
}

MyLoader::~MyLoader(void) {
}

Object* MyLoader::loadFile(const char *fileName) {
	FILE*  input;

	input = fopen(fileName,"rb");

	unsigned facesCount;
	unsigned vertexCount;
	unsigned materialCount;
	unsigned materialStart;
	std::vector<Material> &materials = MaterialManager::instance()->getMaterials();
	materialStart = materials.size();

	ALIGN16 float temp1[4];
	ALIGN16 float temp2[4];

	fread(&vertexCount	 ,1,4,input);
	fread(&facesCount	 ,1,4,input);
	fread(&materialCount ,1,4,input);

	Object *object   = new Object;

	if(materialCount==0)
		object->materials.push_back(0);

	for(unsigned i=0;i<materialCount;i++) {
		object->materials.push_back(materialStart++);
		materials.push_back(Material());
		fread(&materials.back(),sizeof(Material),1,input);
	};

	fread(temp1 ,1,16,input);
	fread(temp2 ,1,16,input);

	object->vertices = new Vertex[vertexCount];
	object->normal	 = new Vertex[vertexCount];
	object->faces	 = new Triangle[facesCount];
	object->vertexCount = vertexCount;
	object->facesCount	= facesCount;

	fread(object->vertices,16,vertexCount,input);
	fread(object->normal  ,16,vertexCount,input);
	fread(object->faces   ,16,facesCount ,input);
	
	fclose(input);

	object->minimum.vec = _mm_load_ps(temp1);
	object->maximum.vec = _mm_load_ps(temp2);

	return object;
};
