/*
**	  GPU Raytrace - Diploma thesis, Czech technical university
**	  Copyright (C) 2008 Martin Zlatuska
**
**	  This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <string.h>

#include <vector_types.h>
#include <vector>

struct Material {
	float3  ambient;
	float3	diffuse;
	float3  specular;
	float   shinines;
	float	reflectivity;

	bool operator==(const Material& other) {
		return memcmp(this,&other,sizeof(Material)) == 0;
	};
};

class MaterialManager {
	static MaterialManager* objInstance;

	std::vector<Material>	materials;
	Material*				videoCopy;

public:
	static MaterialManager*	instance() {
		if(!objInstance) {
			static MaterialManager materialManager;
			objInstance = &materialManager;
			//objInstance=new MaterialManager();
		};
		return objInstance;
	};

	std::vector<Material>& getMaterials() {return materials;};

	Material*	copyToCuda();
	void		clearCuda();	
	void		reset()	{materials.resize(0);};
};
