/*
**	  GPU Raytrace - Diploma thesis, Czech technical university
**	  Copyright (C) 2008 Martin Zlatuska
**
**	  This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <math.h>
#include "Utils/SPMatrix.h"
#include "Utils/SseUtils.h"

//! Coordinate system transformation
class Transformation : public SPMatrix,public AllignedS
{

public:
	//! Creates new identity transformation
	Transformation() {
		IdentityMatrix();
	}

	//! Resets transformation to identity matrix
	void reset() {
		IdentityMatrix();
	};

	//! Append translation transformation
	void translate(float x,float y,float z) {
		SPMatrix translation;
		translation.TranslateMatrix(x,y,z);
		*this *= translation;
	}

	void scale(float x,float y,float z) {
		SPMatrix scaleMatrix;
		scaleMatrix.ScaleMatrix(x,y,z);
		*this *= scaleMatrix;
	};

	//! Append rotation transformation
	void rotate(float x,float y,float z,float angle) {
		float cosAngle=cosf(angle);
		float sinAngle=sinf(angle);
		float oneCosAngle=1-cosAngle;
		float xy=oneCosAngle*x*y;
		float xz=oneCosAngle*x*z;
		float yz=oneCosAngle*y*z;

		SPMatrix rotation;
		rotation._11=cosAngle + oneCosAngle*x*x;
		rotation._12=xy - sinAngle*z;
		rotation._13=xz + sinAngle*y;
		rotation._14=0.0f;

		rotation._21=xy + sinAngle*z;
		rotation._22=cosAngle + oneCosAngle*y*y;
		rotation._23=yz - sinAngle*x;
		rotation._24=0.0f;

		rotation._31=xz - sinAngle*y;
		rotation._32=yz + sinAngle*x;
		rotation._33=cosAngle + oneCosAngle*z*z;
		rotation._34=0.0f;

		rotation._41=0.0f;
		rotation._42=0.0f;
		rotation._43=0.0f;
		rotation._44=1.0f;

		*this *= rotation;
	}
};
