//////////////////////////////////////////////////////////////////////////////
//
// SPMATRIX.H
//
//  Declaration of the SPMatrix, SPVector and the SPVector3 classes.
//
//  Version 1.0
//  Written by Zvi Devir, Intel MSL
//
//////////////////////////////////////////////////////////////////////////////

//
//   Copyright (c) 2001 Intel Corporation.
//
// Permition is granted to use, copy, distribute and prepare derivative works 
// of this library for any purpose and without fee, provided, that the above 
// copyright notice and this statement appear in all copies.  
// Intel makes no representations about the suitability of this library for 
// any purpose, and specifically disclaims all warranties. 
// See LEGAL.TXT for all the legal information.
//

#pragma once

#include <xmmintrin.h> 
#include <emmintrin.h>

#ifdef __GNUC__
    #define ALIGN16 __attribute__ ((aligned (16)))
#endif
#ifdef _MSC_VER
    #define ALIGN16 _CRT_ALIGN(16)
#endif
	

class SPMatrix;
class SPVector;

class SPMatrix {
public:
	union {
        struct {
            __m128 _L1, _L2, _L3, _L4;
        };
        struct {
            float   _11, _12, _13, _14;
            float   _21, _22, _23, _24;
            float   _31, _32, _33, _34;
            float   _41, _42, _43, _44;
        };
    };


	void IdentityMatrix();

	void TranslateMatrix(const float dx, const float dy, const float dz);
    void ScaleMatrix(const float a, const float b, const float c);

	SPMatrix& operator *= (const SPMatrix&);
};

class SPVector {
public:
	union {
        __m128 vec;
        struct { 
            float   x,y,z,w;
        };
    };

	SPVector() {};
	SPVector(const __m128 &m) : vec(m) {};
	SPVector(const float x, const float y, const float z, const float w=1.0f) { vec = _mm_set_ps(w,z,y,x);};

	friend SPVector operator * (const SPVector&, const SPMatrix&);
	friend float	operator * (const SPVector&, const SPVector&);             // Dot Product
	friend SPVector	operator * (const SPVector&, const float);
	friend SPVector operator - (const SPVector&, const SPVector&);
	friend SPVector operator % (const SPVector&, const SPVector&);          // Cross Product
	friend SPVector operator | (const SPVector&, const SPVector&);          // Elements Product

	SPVector& operator *= (const SPMatrix&);
	SPVector& operator *= (const float);
	SPVector& operator += (const SPVector&);
    SPVector& operator -= (const SPVector&);

	SPVector& Normalize();
};

#include "SPMatrix.inl"
