//--maxrregcount=32

/*
**	  GPU Raytrace - Diploma thesis, Czech technical university
**	  Copyright (C) 2008 Martin Zlatuska
**
**	  This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <math.h>
#include <float.h>

#ifdef _WIN32
	#include <windows.h>
#endif

#include <vector_functions.h>

#include <cuda.h>

#include "Camera.h"
#include "SceneLights.h"
#include "Material.h"
#include "FormatImport/PlyLoader.h"
#include "FormatImport/TriLoader.h"
#include "FormatImport/MyLoader.h"
#include "FormatImport/InventorReader.h"
#include "Partitoning/UniformGrid.h"
#include "Partitoning/KdTree.h"
#include "Partitoning/Bvh.h"

#include "Transformations/AnimationPath.h"

#include "Gpu/Wrapper.h"

#include "TestFunctions.h"

const float PI2  = 6.283185307179586476925286766559f;
const float PI2A = 57.295779513082320876798154814105f;
const float STEP = 0.17453292519943295769236907684886f;

class RobotLights {
public:	
	void operator()(SceneLights &lights,float3 &pos) {
		lights.back().possition = make_float3(pos.x,pos.y+20.0f,pos.z);
	};

	void setup(SceneLights &lights) {
		lights.push_back(Light());
		lights.back().kAmbient  = make_float3(0.4f,0.4f,0.4f);
		lights.back().kDiffuse  = make_float3(0.6f,0.6f,0.6f);
	};
};

class MuseumLights {
public:	
	void operator()(SceneLights &lights,float3 &pos) {};

	void setup(SceneLights &lights) {
		lights.push_back(Light());
		lights.back().possition = make_float3(-4.0f,4.5f,0.0f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);
	
		lights.push_back(Light());
		lights.back().possition = make_float3(4.0f,4.5f,0.0f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);
	};
};

class TheatreLights {
public:	
	void operator()(SceneLights &lights,float3 &pos) {};

	void setup(SceneLights &lights) {
		lights.push_back(Light());
		lights.back().possition = make_float3(-20.0f,0.0f,1.5f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);
	
		lights.push_back(Light());
		lights.back().possition = make_float3(  3.3,0.0f,-3.85f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);
	};
};

class ConferenceLights {
public:	
	void operator()(SceneLights &lights,float3 &pos) {};

	void setup(SceneLights &lights) {
		lights.push_back(Light());
		lights.back().possition = make_float3(0.76f,0.0f,1.26f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);
	
		lights.push_back(Light());
		lights.back().possition = make_float3(-2.11,0.0f,1.26f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);
	};
};

class OfficeLights {
public:	
	void operator()(SceneLights &lights,float3 &pos) {};

	void setup(SceneLights &lights) {
		lights.push_back(Light());
		lights.back().possition = make_float3(-1.386864f,1.275630f,-0.824793f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);

		lights.push_back(Light());
		lights.back().possition = make_float3(1.851579f,1.308550f,0.794380f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);
	
		lights.push_back(Light());
		lights.back().possition = make_float3(-1.887669f,-0.30f,0.342509f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);
	};
};

class KitchenLights {
public:	
	void operator()(SceneLights &lights,float3 &pos) {};

	void setup(SceneLights &lights) {
		lights.push_back(Light());

		lights.back().possition = make_float3(2.15f,2.00f,0.90f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);
	
		lights.push_back(Light());
		lights.back().possition = make_float3(2.35f, 2.00f, 0.90f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);

		lights.push_back(Light());
		lights.back().possition = make_float3(2.15f,2.00f,1.10f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);

		lights.push_back(Light());
		lights.back().possition = make_float3(2.35f,2.00f,1.10f);
		lights.back().kAmbient  = make_float3(0.2f,0.2f,0.2f);
		lights.back().kDiffuse  = make_float3(0.4f,0.4f,0.4f);
	};
};


int main(int argc,char **argv) {
	unsigned test;

	printf("Select test\n");
	printf("1-Stanford models\n2-BART models\n3-MgfModels\n");
	int count = scanf("%i",&test);

	if(count!=1 || test==0 || test>3) {
		printf("Wrong option-exiting\n");
		return -1;
	}

	if(test==3) {
		prepareTest("log-MGF.csv");
	
		walkScene("scenes/theatre.iv" ,"scenes/theatre.aff" ,0,-1,true,TheatreLights());
		walkScene("scenes/conference.iv" ,"scenes/conference.aff" ,1,-1,true,ConferenceLights());
		walkScene("scenes/office.iv" ,"scenes/office.aff" ,0,-1,true,OfficeLights());

		endTest();
	} else if(test==2) {
		prepareTest("log-BART.csv");

		walkScene("scenes/robots.mdl" ,"scenes/robots.aff" ,0,128,false,RobotLights());
		walkScene("scenes/museum.mdl" ,"scenes/museum.aff" ,0,-1 ,false,MuseumLights());
		walkScene("scenes/kitchen.mdl","scenes/kitchen.aff",0,128,false,KitchenLights());

		endTest();
	} else {
		prepareTest("log-Simple.csv");

		doSimpleTest("scenes\\bunny.ply"	,"Bunny" ,1.5f);
		doSimpleTest("scenes\\dragon.ply"	,"Dragon",1.1f);
		doSimpleTest("scenes\\buddha.ply"	,"Budha" ,1.1f);

		endTest();
	};

//#define STRUCTSTATS
#ifdef STRUCTSTATS
	structStats("scenes//bunny.ply","Bunny");
	structStats("scenes//dragon.ply","Dragon");
	structStats("scenes//buddha.ply","Budha");

	structStats("scenes//robots.mdl","Robots",128);
	structStats("scenes//museum.mdl","Museum");
	structStats("scenes//kitchen.mdl","Kitchen",128);
	
	structStats("scenes//theatre.iv","Theatre");
	structStats("scenes//office.iv","Office");
	structStats("scenes//conference.iv","Conference");
#endif
	return 0;
};
